containers v0.4.0 Containers.Sequenceable protocol
provides next function to chain compuations together
If a custome structure implements a field value in the struct,
then that structe can derive Sequenceable.
Summary
Functions
struct argument is some sturct that implements Sequenceable
Types
Functions
next(struct, f)
next(Containers.sequenceable, (any -> Containers.sequenceable)) :: Containers.sequenceable
struct argument is some sturct that implements Sequenceable
f is a function that takes in the inner value potentially does something
to the inner value and wraps it back up in a struct